#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#    Name:	Component Array
#      By:	sdmitch
#   Usage:	Place selected component on a equilateral triangular grid or rectangular grid
#	 	 Note:	Faces do not have to be on the XY plane.  They can be tilted.
#			
#    Date:	Sep 2011
#------------------------------------------------------------------------------------------------
#
require 'Sketchup'
#
filename=File.basename(__FILE__)
#
if not file_loaded?(filename)
    UI.menu('Plugins').add_item('Component Array') { SDM_CA.component_array }
end
#
file_loaded(filename)
#
module SDM_CA

	def self.component_array
		mod = Sketchup.active_model
		@ent = mod.active_entities
		sel = mod.selection
		@Spacing = 10.m if !@Spacing
		@Angle = 0 if @Angle
		@G_type = "Rec" if !@G_type
		@Orient = "Vert" if !@Orient
		@face=nil
		@face=sel.first if sel.first.is_a?(Sketchup::Face)
		if @face
			cd=[];@ent.each{|d| cd << d.definition if d.is_a?(Sketchup::ComponentInstance)}
			cn=[];cd.each{|d| cn << d.name}
			comps = cn.join("|");@C_name=cn[-1] if !@C_name || !comps.include?(@C_name)
			prompts=["Component:","Grid Spacing:","Grid Angle:","Grid Type:","Orientation:"]
			defaults=[@C_name,@Spacing,@Angle,@G_type,@Orient]
			options=[comps,nil,nil,"Rec|Tri","Vert|Perp"]
			inp_data=inputbox(prompts,defaults,options,"Component Array")
			if !inp_data then;return;end
			@C_name=inp_data[0].to_s;ci=cn.index(@C_name);@Comp=cd[ci];
			@Spacing=inp_data[1].to_l;@Angle=inp_data[2].to_i;@G_type=inp_data[3];@Orient=inp_data[4]
			pl=@face.plane;ctr=@face.bounds.center; norm=@face.normal; norm.reverse! if norm.z < 0; 
			ctr=ctr.project_to_plane pl if !ctr.on_plane? pl; @xaxis,@yaxis,@zaxis=norm.axes
			tr = Geom::Transformation.rotation(ctr,norm,@Angle.degrees); @zaxis=Z_AXIS if @Orient=="Vert"
			@xaxis.transform! tr; @yaxis.transform! tr; maxd = @face.bounds.diagonal*2
			@pt0=ctr.offset(@xaxis.reverse,maxd/2).offset(@yaxis.reverse,maxd/2)
			nx=(maxd/@Spacing).ceil;ny=(maxd/@Spacing).ceil
			mod.start_operation "Component Array", true
			case @G_type
				when "Rec";self.rectangular_array(nx,ny)
				when "Tri";self.triangular_array(nx,ny)
			end
			mod.commit_operation
		else
			UI.messagebox "No face pre-selected"
		end
	end
	
	def self.rectangular_array(nx,ny)
		for i in 0..ny
			pt1=@pt0.clone
			for j in 0..nx
				if @face.classify_point(pt1)==1
					# @ent.add_cpoint(pt1)
					tr=Geom::Transformation.new pt1,@zaxis
					@ent.add_instance(@Comp,tr)
				end
				pt1.offset!(@xaxis,@Spacing);
			end 
			@pt0.offset!(@yaxis,@Spacing)
		end
	end
	
	def self.triangular_array(nx,ny)
			odd_row=true
			for i in 0..ny
				pt1=@pt0.clone
				for j in 0..nx
					if @face.classify_point(pt1)==1
						# @ent.add_cpoint(pt1)
						tr=Geom::Transformation.new pt1,@zaxis
						@ent.add_instance(@Comp,tr)
					end
					pt1.offset!(@xaxis,@Spacing);
				end 
				@pt0.offset!(@yaxis,@Spacing*0.866);#puts"next @pt0=#{@pt0}"
				if odd_row
					@pt0.offset!(@xaxis,@Spacing*0.5);odd_row=false
				else
					@pt0.offset!(@xaxis,-@Spacing*0.5);odd_row=true
				end
			end
	end
end